@@ -809,10 +809,12 @@ class RandomNoiseDataset(Dataset):
 
 class TextSequenceDataset(Dataset):
     def __init__(self, corpus=None, is_onehot=False, sequence_offset=0, storage_unit='section',section_delimiter='\n\n', stopwords=None, sequence_length: int = 64, sequence_start_at='random',
-                 object_type=ObjectType.corpus, symbol=None, **kwargs):
+                 include_segment_ids=False,include_mask_ids=False,object_type=ObjectType.corpus, symbol=None, **kwargs):
         super().__init__(None,symbol=symbol, object_type=object_type, **kwargs)
         self.sequence_length = sequence_length
         self.sequence_start_at = sequence_start_at
+        self.length_index = OrderedDict()
+        self.vocabs_frequency = OrderedDict()
         self.transform_funcs = []
         if len(section_delimiter) == 2:
             self.section_delimiter = section_delimiter
@@ -829,6 +831,7 @@ class TextSequenceDataset(Dataset):
         self.sequence_offset = sequence_offset
         self.dtype = np.float32 if self.is_onehot else np.int64
 
+
         self.is_paired_process = False
 
 
@@ -836,6 +839,7 @@ class TextSequenceDataset(Dataset):
         if hasattr(corpus, "__iter__"):
             self.items.extend(corpus)
             if self.vocabs is None:
+
                 chars = sorted(list(set( ''.join(corpus ))))
                 chars.insert(0, '[CLS]')
                 chars.insert(1, '[SEP]')
@@ -846,6 +850,10 @@ class TextSequenceDataset(Dataset):
                 self.vocabs = chars
                 self.text2index = dict((c, i) for i, c in enumerate(chars))
                 self.index2text = dict((i, c) for i, c in enumerate(chars))
+                self.vocabs_frequency=OrderedDict((c, 1) for i, c in enumerate(chars))
+                for ch in list(''.join(corpus)):
+                    self.vocabs_frequency[ch] += 1
+
             else:
                 new_chars = sorted(list(set(''.join(corpus))))
                 new_chars=[ch for ch in new_chars if ch not in self.vocabs]
@@ -854,6 +862,11 @@ class TextSequenceDataset(Dataset):
                     self.vocabs.append(ch)
                     self.text2index[ch]=len(self.vocabs)-1
                     self.index2text[len(self.vocabs)-1]=ch
+                    self.vocabs_frequency[ch]=1
+
+                for ch in list(''.join(corpus)):
+                    self.vocabs_frequency[ch] += 1
+
             self.length_index=OrderedDict()
             total_len=0
             for i in range(len(self.items)):
@@ -883,7 +896,6 @@ class TextSequenceDataset(Dataset):
     def __getitem__(self, index: int):
         sequencetext = None
         if isinstance(self.sequence_offset, int):
-
             if self.sequence_start_at == 'random':
                 for k,v in self.length_index.item_list:
                     if v>index:
@@ -917,16 +929,9 @@ class TextSequenceDataset(Dataset):
                 sectiontext.insert(0, '[CLS]')
                 sectiontext.append('[SEP]')
                 sequencetext = sectiontext[self.sequence_offset:builtins.min(self.sequence_offset + self.sequence_length,len(sectiontext))]
+
         return sequencetext
 
-        if self.is_paired_process == False and len(self.transform_funcs) == 0:
-            return text_backend_adaption(arr)
-        elif not self.is_paired_process:
-            return self.text_transform(arr)
-        elif self.is_paired_process:
-            return arr
-
-        return None
 
     def data_transform(self, text_data):
         if len(self.transform_funcs) == 0:
@@ -937,7 +942,9 @@ class TextSequenceDataset(Dataset):
         #text_data = text_backend_adaption(text_data)
 
         arr = None
-        if self.is_onehot:
+        if isinstance(text_data,np.ndarray):
+            return text_data
+        elif self.is_onehot:
             arr = np.zeros((self.sequence_length, len(self.text2index)))
             for i in range(self.sequence_length):
                 if i < len(text_data):
