@@ -1,34 +1,22 @@
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
+
 import base64
-import glob
-import glob
-import gzip
+import datetime
 import hashlib
 import json
 import os
 import platform
 import shutil
-import subprocess
 import sys
 import tarfile
-import threading
 import urllib.request
-import warnings
 import zipfile
-from sys import stderr
-import datetime
-import numpy as np
-import requests
+
 import six
-import tqdm
 from scipy.io import loadmat
-from six.moves.urllib.error import HTTPError
-from six.moves.urllib.error import URLError
-from six.moves.urllib.request import urlopen
 from tqdm import tqdm
-from urllib3.exceptions import NewConnectionError
 
 from trident.backend.common import *
 
@@ -37,17 +25,19 @@ try:
 except ImportError:
     from six.moves.urllib.request import urlretrieve
 
+
 def is_connected():
     try:
-        urllib.request.urlopen('https://google.com') #Python 3.x
+        urllib.request.urlopen('https://google.com')  # Python 3.x
         return True
-    except :
+    except:
         return False
 
 
 def ensure_dir(dirpath):
     if not os.path.exists(dirpath): os.makedirs(dirpath)
 
+
 def ensure_parent_dir(childpath):
     ensure_dir(os.path.dirname(childpath))
 
@@ -68,6 +58,7 @@ class TqdmProgress(tqdm):
             self.total = tsize
         self.update(b * bsize - self.n)  # will also set self.n = b * bsize
 
+
 def calculate_md5(fpath, chunk_size=1024 * 1024):
     """Calculate whether the file 's md5 hash is the same as given md5 .
 
@@ -85,19 +76,18 @@ def calculate_md5(fpath, chunk_size=1024 * 1024):
     return md5.hexdigest()
 
 
-
-
 def check_integrity(fpath, md5=None):
     if not os.path.isfile(fpath):
         return False
     if md5 is None:
         return True
-    fmd5=calculate_md5(fpath)
-    return  md5 == fmd5
+    fmd5 = calculate_md5(fpath)
+    return md5 == fmd5
 
-def _write_h(dirname,is_downloaded=False,is_extracted=False,tag=None):
+
+def _write_h(dirname, is_downloaded=False, is_extracted=False, tag=None):
     _h_path = os.path.join(dirname, 'status.json')
-    _h={}
+    _h = {}
     if os.path.exists(_h_path):
         try:
             with open(_h_path) as f:
@@ -105,11 +95,11 @@ def _write_h(dirname,is_downloaded=False,is_extracted=False,tag=None):
         except ValueError:
             _h = {}
     if tag is None:
-        _h['is_downloaded']=is_downloaded
+        _h['is_downloaded'] = is_downloaded
         _h['is_extracted'] = is_extracted
     else:
         if tag not in _h:
-            _h[tag]={}
+            _h[tag] = {}
         _h[tag]['is_downloaded'] = is_downloaded
         _h[tag]['is_extracted'] = is_extracted
 
@@ -119,6 +109,8 @@ def _write_h(dirname,is_downloaded=False,is_extracted=False,tag=None):
     except IOError:
         # Except permission denied.
         pass
+
+
 def _read_h(dirname):
     _h = {}
     _h_path = os.path.join(dirname, 'status.json')
@@ -129,17 +121,18 @@ def _read_h(dirname):
         except ValueError:
             _h = {}
     return _h
-def _delete_h(dirname):
 
+
+def _delete_h(dirname):
     _h_path = os.path.join(dirname, 'status.json')
     if os.path.exists(_h_path):
         os.remove(_h_path)
 
-def download_file(src, dirname, filename, desc=''):
 
+def download_file(src, dirname, filename, desc=''):
     _h = _read_h(dirname)
 
-    is_downloaded= _h[filename].get('is_downloaded', False) if filename in _h else _h.get('is_downloaded', False)
+    is_downloaded = _h[filename].get('is_downloaded', False) if filename in _h else _h.get('is_downloaded', False)
 
     if os.path.exists(os.path.join(dirname, filename)) and _h != {} and is_downloaded == True:
         print('archive file is already existing, donnot need download again.')
@@ -149,18 +142,17 @@ def download_file(src, dirname, filename, desc=''):
             os.remove(os.path.join(dirname, filename))
 
         try:
-            with TqdmProgress(unit='B', unit_scale=True,  leave=True,miniters=10, desc=desc) as t:  # all optional kwargs
+            with TqdmProgress(unit='B', unit_scale=True, leave=True, miniters=10, desc=desc) as t:  # all optional kwargs
                 urlretrieve(src, filename=os.path.join(dirname, filename), reporthook=t.update_to, data=None)
-                _write_h(dirname, True, False,tag=filename)
+                _write_h(dirname, True, False, tag=filename)
             return True
         except Exception as e:
-            _write_h(dirname, False, False,tag=filename)
-            print('***Cannot download data,.\n',flush=True)
+            _write_h(dirname, False, False, tag=filename)
+            print('***Cannot download data,.\n', flush=True)
             print(e)
             return False
 
 
-
 def get_onedrive_directdownload(onedrive_link):
     """
 
@@ -177,11 +169,12 @@ def get_onedrive_directdownload(onedrive_link):
 
     """
     data_bytes64 = base64.b64encode(bytes(onedrive_link, 'utf-8'))
-    data_bytes64_String = data_bytes64.decode('utf-8').replace('/','_').replace('+','-').rstrip("=")
+    data_bytes64_String = data_bytes64.decode('utf-8').replace('/', '_').replace('+', '-').rstrip("=")
     resultUrl = f"https://api.onedrive.com/v1.0/shares/u!{data_bytes64_String}/root/content"
     return resultUrl
 
-def download_file_from_google_drive(file_id, dirname, filename=None, md5=None,need_up_to_date=False):
+
+def download_file_from_google_drive(file_id, dirname, filename=None, md5=None, need_up_to_date=False):
     """Download a Google Drive file from  and place it in root.
 
     Args:
@@ -200,9 +193,9 @@ def download_file_from_google_drive(file_id, dirname, filename=None, md5=None,ne
     _h = _read_h(dirname)
 
     dest_path = os.path.join(dirname, filename)
-    is_downloaded= _h[filename].get('is_downloaded', False) if filename in _h else _h.get('is_downloaded', False)
+    is_downloaded = _h[filename].get('is_downloaded', False) if filename in _h else _h.get('is_downloaded', False)
 
-    if os.path.exists(dest_path) and os.path.isfile(dest_path) and _h != {} and is_downloaded== True and need_up_to_date==False:
+    if os.path.exists(dest_path) and os.path.isfile(dest_path) and _h != {} and is_downloaded == True and need_up_to_date == False:
         print('archive file is already existing, donnot need download again.')
         return True
     else:
@@ -215,15 +208,18 @@ def download_file_from_google_drive(file_id, dirname, filename=None, md5=None,ne
                 params = {'id': file_id, 'confirm': token}
                 response = session.get(url, params=params, stream=True)
             _save_response_content(response, dest_path)
-            _write_h(dirname,True,False,filename)
+            _write_h(dirname, True, False, filename)
             return True
         except Exception as e:
             _write_h(dirname, False, False, filename)
             print('***Cannot download data, so the data provider cannot initialized.\n', flush=True)
-            print( '***Please check your internet or download  files from following url in another computer, \n and then put them into {0}\n {1} '.format(dirname, 'https://drive.google.com/open?id={0}'.format(file_id)), flush=True)
+            print('***Please check your internet or download  files from following url in another computer, \n and then put them into {0}\n {1} '.format(dirname,
+                                                                                                                                                         'https://drive.google.com/open?id={0}'.format(
+                                                                                                                                                             file_id)), flush=True)
             print(e)
             return False
 
+
 def download_file_from_onedrive(onedrive_path, dirname, filename=None, md5=None):
     """Download a OneDrive file from  and place it in root.
 
@@ -245,12 +241,11 @@ def download_file_from_onedrive(onedrive_path, dirname, filename=None, md5=None)
 
     """
     # Based on https://stackoverflow.com/questions/38511444/python-download-files-from-google-drive-using-url
-    import requests
-    url=get_onedrive_directdownload(onedrive_path)
+    url = get_onedrive_directdownload(onedrive_path)
     # if os.path.exists(os.path.join(dirname, filename)):
     #     print('archive file is already existing, donnot need download again.')
-    dest_path=os.path.join(os.path.join(get_trident_dir(),'models'), filename)
-    if os.path.exists(dest_path) and os.path.isfile(dest_path) and (datetime.datetime.now()-get_file_modified_time(dest_path)).seconds<12*60*60:
+    dest_path = os.path.join(os.path.join(get_trident_dir(), 'models'), filename)
+    if os.path.exists(dest_path) and os.path.isfile(dest_path) and (datetime.datetime.now() - get_file_modified_time(dest_path)).seconds < 12 * 60 * 60:
         print('archive file is already existing, donnot need download again.')
         return True
     else:
@@ -294,13 +289,13 @@ def get_image_from_google_drive(file_id):
     make_dir_if_need(dirname)
     fpath = os.path.join(dirname, filename)
 
-    if os.path.exists(fpath) :
+    if os.path.exists(fpath):
         os.remove(fpath)
     try:
         session = requests.Session()
         response = session.get(url, params={'id': file_id}, stream=True)
-        content_type=response.headers.get('content-type')
-        filename=file_id+'.'+content_type.split('/')[-1]
+        content_type = response.headers.get('content-type')
+        filename = file_id + '.' + content_type.split('/')[-1]
         fpath = os.path.join(dirname, filename)
 
         token = _get_confirm_token(response)
@@ -311,12 +306,14 @@ def get_image_from_google_drive(file_id):
         return fpath
     except Exception as e:
         print('***Cannot download data, so the data provider cannot initialized.\n', flush=True)
-        print( '***Please check your internet or download  files from following url in another computer, \n and then put them into {0}\n {1} '.format(dirname, 'https://drive.google.com/open?id={0}'.format(file_id)), flush=True)
+        print('***Please check your internet or download  files from following url in another computer, \n and then put them into {0}\n {1} '.format(dirname,
+                                                                                                                                                     'https://drive.google.com/open?id={0}'.format(
+                                                                                                                                                         file_id)), flush=True)
         print(e)
         return None
 
 
-def get_file_from_google_drive(file_name,file_id):
+def get_file_from_google_drive(file_name, file_id):
     """Download a Google Drive image  and place it in root.
 
     Args:
@@ -362,6 +359,7 @@ def get_file_from_google_drive(file_name,file_id):
         return None
     return fpath
 
+
 def download_model_from_google_drive(file_id, dirname, filename=None, md5=None):
     """Download a Google Drive file from  and place it in root.
 
@@ -383,16 +381,17 @@ def download_model_from_google_drive(file_id, dirname, filename=None, md5=None):
     need_download = True
     check_internet = None
     try:
-        if os.path.exists(os.path.join(dirname, 'models_md5.json')) and os.path.isfile(os.path.join(dirname, 'models_md5.json')) and (datetime.datetime.now() - get_file_modified_time(os.path.join(dirname, 'models_md5.json'))).seconds < 24 * 60 * 60:
+        if os.path.exists(os.path.join(dirname, 'models_md5.json')) and os.path.isfile(os.path.join(dirname, 'models_md5.json')) and (
+                datetime.datetime.now() - get_file_modified_time(os.path.join(dirname, 'models_md5.json'))).seconds < 24 * 60 * 60:
             with open(os.path.join(dirname, 'models_md5.json')) as f:
                 models_md5 = json.load(f)
         else:
             session = requests.Session()
             response = session.get(url, params={'id': '12XLjt9Zcaoo90WGG6R5N0U6Sf_KBZZn_'}, stream=False)
-            if response.status_code==200:
+            if response.status_code == 200:
                 if os.path.exists(os.path.join(dirname, 'models_md5.json')):
                     os.remove(os.path.join(dirname, 'models_md5.json'))
-                with open(os.path.join(dirname, 'models_md5.json'),"wb") as f:
+                with open(os.path.join(dirname, 'models_md5.json'), "wb") as f:
                     f.write(response.content)
 
             check_internet = True
@@ -402,8 +401,7 @@ def download_model_from_google_drive(file_id, dirname, filename=None, md5=None):
     except Exception as e:
         print(e)
         PrintException()
-        check_internet=False
-
+        check_internet = False
 
     if check_internet == False:
         if os.path.exists(os.path.join(dirname, filename)):
@@ -412,7 +410,7 @@ def download_model_from_google_drive(file_id, dirname, filename=None, md5=None):
         else:
             print('***Cannot download data, so the data provider cannot initialized.\n', flush=True)
             print('***Please check your internet or download  files from following url in another computer, \n and then '
-                'put them into {0}\n {1} '.format( dirname, 'https://drive.google.com/open?id={0}'.format(file_id)), flush=True)
+                  'put them into {0}\n {1} '.format(dirname, 'https://drive.google.com/open?id={0}'.format(file_id)), flush=True)
 
         return False
     else:
@@ -448,12 +446,13 @@ def download_model_from_google_drive(file_id, dirname, filename=None, md5=None):
             print(e)
             print('***Cannot download data, so the data provider cannot initialized.\n', flush=True)
             print('***Please check your internet or download  files from following url in another computer, \n and then '
-                'put them into {0}\n {1} '.format(dirname, 'https://drive.google.com/open?id={0}'.format(file_id)), flush=True)
+                  'put them into {0}\n {1} '.format(dirname, 'https://drive.google.com/open?id={0}'.format(file_id)), flush=True)
             print(e)
 
         return False
 
-#https://1drv.ms/u/s!AsqOV38qroofiZrqNAQvo2CuX_cyWQE?e=Aa8v7D
+
+# https://1drv.ms/u/s!AsqOV38qroofiZrqNAQvo2CuX_cyWQE?e=Aa8v7D
 def download_model_from_onedrive(onedrive_path, dirname, filename=None, md5=None):
     """Download a Google Drive file from  and place it in root.
 
@@ -464,8 +463,7 @@ def download_model_from_onedrive(onedrive_path, dirname, filename=None, md5=None
         md5 (str, optional): MD5 checksum of the download. If None, do not check
     """
     # Based on https://stackoverflow.com/questions/38511444/python-download-files-from-google-drive-using-url
-    import requests
-    url=get_onedrive_directdownload(onedrive_path)
+    url = get_onedrive_directdownload(onedrive_path)
 
     fpath = os.path.join(dirname, filename)
     isload = False
@@ -473,11 +471,12 @@ def download_model_from_onedrive(onedrive_path, dirname, filename=None, md5=None
     need_download = True
     check_internet = None
     try:
-        if os.path.exists(os.path.join(dirname, 'models_md5.json')) and os.path.isfile(os.path.join(dirname, 'models_md5.json')) and (datetime.datetime.now() - get_file_modified_time(os.path.join(dirname, 'models_md5.json'))).seconds < 24 * 60 * 60:
+        if os.path.exists(os.path.join(dirname, 'models_md5.json')) and os.path.isfile(os.path.join(dirname, 'models_md5.json')) and (
+                datetime.datetime.now() - get_file_modified_time(os.path.join(dirname, 'models_md5.json'))).seconds < 24 * 60 * 60:
             with open(os.path.join(dirname, 'models_md5.json')) as f:
                 models_md5 = json.load(f)
         else:
-            download_file_from_onedrive("https://1drv.ms/u/s!AsqOV38qroofiZrqNAQvo2CuX_cyWQE?e=Aa8v7D",dirname,'models_md5.json')
+            download_file_from_onedrive("https://1drv.ms/u/s!AsqOV38qroofiZrqNAQvo2CuX_cyWQE?e=Aa8v7D", dirname, 'models_md5.json')
 
             check_internet = True
             if os.path.exists(os.path.join(dirname, 'models_md5.json')):
@@ -486,9 +485,7 @@ def download_model_from_onedrive(onedrive_path, dirname, filename=None, md5=None
     except Exception as e:
         print(e)
         PrintException()
-        check_internet=False
-
-
+        check_internet = False
 
     if check_internet == False:
         if os.path.exists(os.path.join(dirname, filename)):
@@ -498,7 +495,7 @@ def download_model_from_onedrive(onedrive_path, dirname, filename=None, md5=None
             _write_h(dirname, False, False, filename)
             print('***Cannot download data, so the data provider cannot initialized.\n', flush=True)
             print('***Please check your internet or download  files from following url in another computer, \n and then '
-                'put them into {0}\n {1} '.format( dirname,url), flush=True)
+                  'put them into {0}\n {1} '.format(dirname, url), flush=True)
 
         return False
     else:
@@ -529,13 +526,12 @@ def download_model_from_onedrive(onedrive_path, dirname, filename=None, md5=None
             print(e)
             print('***Cannot download data, so the data provider cannot initialized.\n', flush=True)
             print('***Please check your internet or download  files from following url in another computer, \n and then '
-                'put them into {0}\n {1} '.format(dirname, url), flush=True)
+                  'put them into {0}\n {1} '.format(dirname, url), flush=True)
             print(e)
 
         return False
 
 
-
 def _get_confirm_token(response):
     for key, value in response.cookies.items():
         if key.startswith('download_warning'):
@@ -557,70 +553,67 @@ def _save_response_content(response, destination, chunk_size=32768):
         pbar.close()
 
 
-
-
-
 def extract_archive(file_path, target_folder=None, archive_format='auto'):
-  """Extracts an archive if it matches tar, tar.gz, tar.bz, or zip formats.
+    """Extracts an archive if it matches tar, tar.gz, tar.bz, or zip formats.
 
-  Args:
-      file_path: path to the archive file
-      target_folder: path to extract the archive file
-      archive_format: Archive format to try for extracting the file.
-          Options are 'auto', 'tar', 'zip', and None.
-          'tar' includes tar, tar.gz, and tar.bz files.
-          The default 'auto' is ['tar', 'zip'].
-          None or an empty list will return no matches found.
+    Args:
+        file_path: path to the archive file
+        target_folder: path to extract the archive file
+        archive_format: Archive format to try for extracting the file.
+            Options are 'auto', 'tar', 'zip', and None.
+            'tar' includes tar, tar.gz, and tar.bz files.
+            The default 'auto' is ['tar', 'zip'].
+            None or an empty list will return no matches found.
 
-  Returns:
-      True if a match was found and an archive extraction was completed,
-      False otherwise.
-  """
-  folder,file, ext=split_path(file_path)
-  _h = _read_h(target_folder)
-  filename=file+ext
-  is_extracted = _h[filename].get('is_extracted', False) if filename in _h else _h.get('is_extracted', False)
-  if _h != {} and is_extracted== True and os.path.exists(file_path) :
-      print('extraction is finished, donnot need extract again.')
-      return True
-  if archive_format is None:
-    return False
-  if archive_format == 'auto':
-    archive_format = ['tar', 'zip']
-  if isinstance(archive_format, six.string_types):
-    archive_format = [archive_format]
-
-  is_match_fn = tarfile.is_tarfile
-  open_fn = tarfile.open
-  for archive_type in archive_format:
-    if archive_type == 'tar':
-      open_fn = tarfile.open
-      is_match_fn = tarfile.is_tarfile
-    if archive_type == 'zip':
-      open_fn = zipfile.ZipFile
-      is_match_fn = zipfile.is_zipfile
-
-    if is_match_fn(file_path):
-        print('Starting to decompress the archive....')
-        with open_fn(file_path) as archive:
-            try:
-                archive.extractall(target_folder)
-                _write_h(target_folder, True, True)
-            except (tarfile.TarError, RuntimeError, KeyboardInterrupt):
-                sys.stderr.write('Decompressing the archive is not success')
-                PrintException()
-                _write_h(target_folder, True, False,filename)
-                if os.path.exists(target_folder):
-                    shutil.rmtree(target_folder)
-                raise
-        _write_h(target_folder, True, True, filename)
+    Returns:
+        True if a match was found and an archive extraction was completed,
+        False otherwise.
+    """
+    folder, file, ext = split_path(file_path)
+    _h = _read_h(target_folder)
+    filename = file + ext
+    is_extracted = _h[filename].get('is_extracted', False) if filename in _h else _h.get('is_extracted', False)
+    if _h != {} and is_extracted == True and os.path.exists(file_path):
+        print('extraction is finished, donnot need extract again.')
         return True
-    else:
-        _write_h(target_folder, True, False,filename)
-  return False
+    if archive_format is None:
+        return False
+    if archive_format == 'auto':
+        archive_format = ['tar', 'zip']
+    if isinstance(archive_format, six.string_types):
+        archive_format = [archive_format]
+
+    is_match_fn = tarfile.is_tarfile
+    open_fn = tarfile.open
+    for archive_type in archive_format:
+        if archive_type == 'tar':
+            open_fn = tarfile.open
+            is_match_fn = tarfile.is_tarfile
+        if archive_type == 'zip':
+            open_fn = zipfile.ZipFile
+            is_match_fn = zipfile.is_zipfile
+
+        if is_match_fn(file_path):
+            print('Starting to decompress the archive....')
+            with open_fn(file_path) as archive:
+                try:
+                    archive.extractall(target_folder)
+                    _write_h(target_folder, True, True)
+                except (tarfile.TarError, RuntimeError, KeyboardInterrupt):
+                    sys.stderr.write('Decompressing the archive is not success')
+                    PrintException()
+                    _write_h(target_folder, True, False, filename)
+                    if os.path.exists(target_folder):
+                        shutil.rmtree(target_folder)
+                    raise
+            _write_h(target_folder, True, True, filename)
+            return True
+        else:
+            _write_h(target_folder, True, False, filename)
+    return False
 
 
-def pickle_it(file_path,obj):
+def pickle_it(file_path, obj):
     """Pickle the obj
 
     Args:
@@ -631,12 +624,36 @@ def pickle_it(file_path,obj):
     with open(file_path, 'wb') as handle:
         pickle.dump(obj, handle, protocol=pickle.HIGHEST_PROTOCOL)
 
+
 def unpickle(file):
     import _pickle as pickle
     with open(file, 'rb') as fo:
         dict = pickle.load(fo, encoding='bytes')
     return dict
 
+def save_dict_as_h5(save_path,dict_need_save):
+    try:
+        import h5py
+    except ImportError:
+        h5py = None
+    if h5py is not None:
+        with h5py.File(save_path, 'w') as f:
+            for k, v in dict_need_save.items():
+                f.create_dataset(k, data=v)
+
+def read_dict_from_h5(save_path):
+    try:
+        import h5py
+    except ImportError:
+        h5py = None
+    if h5py is not None:
+        return_dict=OrderedDict()
+        with h5py.File(save_path, 'r') as f:
+            for k,v in f.items():
+                return_dict[k]=v
+            return return_dict
+
+
 def open_pickle(fpath, data_key='data', label_key='labels'):
     d = unpickle(fpath)
     d_decoded = {}
@@ -654,12 +671,11 @@ def check_image(image, imagepath):
         return (hashlib.md5(fin.read()).hexdigest() == image['md5'])
 
 
-
-
 def read_mat(mat_path):
     mat = loadmat(mat_path)
     return mat
 
+
 def get_file_create_time(file_path):
     if platform.system() == 'Windows':
         return os.path.getctime(file_path)
@@ -671,4 +687,3 @@ def get_file_create_time(file_path):
             # We're probably on Linux. No easy way to get creation dates here,
             # so we'll settle for when its content was last modified.
             return stat.st_mtime
-
