@@ -190,12 +190,17 @@ class RNNBase(Layer):
 
             # Note: no_grad() is necessary since _cudnn_rnn_flatten_weight is
             # an inplace operation on self._flat_weights
+
             with torch.no_grad():
                 if torch._use_cudnn_rnn_flatten_weight():
+                    num_weights = 4 if self.use_bias else 2
+                    if self.proj_size > 0:
+                        num_weights += 1
                     torch._cudnn_rnn_flatten_weight(
-                        self._flat_weights, (4 if self.use_bias else 2),
-                        self.input_filters, rnn.get_cudnn_mode(self.mode), self.hidden_size, self.num_layers,
-                        self.batch_first, bool(self.bidirectional))
+                        self._flat_weights, num_weights,
+                        self.input_size, rnn.get_cudnn_mode(self.mode),
+                        self.input_filters, self.proj_size, self.num_layers,  # type: ignore
+                        self.batch_first, bool(self.bidirectional))  # type: ignore
 
     def _apply(self, fn):
         ret = super(RNNBase, self)._apply(fn)
@@ -841,10 +846,17 @@ class GRU(RNNBase):
         >>> output, hn = rnn(input, h0)
     """
 
-    def __init__(self, *args, **kwargs):
-        super(GRU, self).__init__('GRU', *args, **kwargs)
-        self.hidden_state = None
-        self.stateful=kwargs.get('stateful',False)
+    def __init__(self, hidden_size,num_layers:int =2,activation=None,stateful=False,use_bias=False,use_attention=False,attention_size=16,batch_first=False,dropout_rate=0,bidirectional=False,name=None,keep_output=False, in_sequence=True,filter_index=-1, **kwargs):
+        super(GRU, self).__init__(mode='GRU', hidden_size=hidden_size,
+                 num_layers=num_layers, stateful=stateful,use_bias=use_bias, batch_first=batch_first,
+                 dropout_rate=dropout_rate, bidirectional=bidirectional,name=name,keep_output=keep_output,in_sequence=in_sequence,filter_index=filter_index)
+
+        self.mode = 'GRU'
+        self.use_attention=use_attention
+        self.attention_size=attention_size
+        self.hidden_state=None
+        self.stateful=stateful
+
 
     def initial_state(self,input) :
         max_batch_size = input.size(0) if self.batch_first else input.size(1)
@@ -867,6 +879,11 @@ class GRU(RNNBase):
         pass
 
     def forward(self,x):  # noqa: F811
+        x = unpack_singleton(x)
+        if isinstance(x, tuple):
+            x, hx = x
+        orig_input = x
+        self.flatten_parameters()
         orig_input = x
         # xxx: isinstance check needs to be in conditional for TorchScript to compile
         if isinstance(orig_input, PackedSequence):
@@ -874,7 +891,7 @@ class GRU(RNNBase):
             max_batch_size = batch_sizes[0]
             max_batch_size = int(max_batch_size)
         else:
-            if self.batch_first == False:
+            if not self.batch_first:
                 x = x.permute(1, 0, 2)
             batch_sizes = None
             max_batch_size = x.size(0) if self.batch_first else x.size(1)
@@ -903,6 +920,6 @@ class GRU(RNNBase):
             output_packed = PackedSequence(output, batch_sizes, sorted_indices, unsorted_indices)
             return output_packed, self.permute_hidden(self.hidden_state, unsorted_indices)
         else:
-            if self.batch_first == False:
+            if not self.batch_first:
                 x = x.permute(1, 0, 2)
             return output, self.permute_hidden(self.hidden_state, unsorted_indices)
