@@ -1,9 +1,13 @@
+import functools
+import operator
+from abc import ABC, abstractmethod
 
-class BaseLogger(object):
+class BaseLogger(ABC):
     """
     Base logger handler. See implementations: TensorboardLogger, VisdomLogger, PolyaxonLogger, MLflowLogger, ...
 
     """
+
     def log_metrics(self, metrics, step):
 
         """Record metrics.
