@@ -22,11 +22,12 @@ from matplotlib.collections import PolyCollection
 from torch._six import container_abcs
 from torch.nn import init
 from torch.nn.parameter import Parameter
+from trident.backend.opencv_backend import image2array
 
 from trident.backend.common import *
 from trident.backend.tensorspec import *
 from trident.backend.pytorch_backend import *
-from trident.backend.pytorch_backend import to_numpy, to_tensor, Layer, Sequential, Combine, load,get_device
+from trident.backend.pytorch_backend import to_numpy, to_tensor, Layer, Sequential, Combine, load,get_device,fix_layer
 from trident.backend.pytorch_ops import *
 from trident.data.bbox_common import clip_boxes_to_image, nms
 from trident.data.image_common import *
@@ -38,7 +39,7 @@ from trident.layers.pytorch_normalizations import get_normalization
 from trident.layers.pytorch_pooling import *
 from trident.optims.pytorch_trainer import *
 from trident.optims.pytorch_trainer import ImageDetectionModel
-
+from trident.data.vision_transforms import Resize,Normalize
 __all__ = ['Pnet','Rnet','Onet','Mtcnn']
 
 _session = get_session()
@@ -66,10 +67,11 @@ def p_net():
     PRelu(num_parameters=1),
     Conv2d((3,3),32,strides=1,auto_pad=False,use_bias=True,name='conv3'),
     PRelu(num_parameters=1),
-    Combine(
-        Conv2d((1,1),1,strides=1,auto_pad=False,use_bias=True,activation='sigmoid',name='conv4_1'),
-        Conv2d((1,1),4,strides=1,auto_pad=False,use_bias=True,name='conv4_2'),
-        Conv2d((1,1),10,strides=1,auto_pad=False,use_bias=True,name='conv4_3')),name='pnet')
+    ModuleDict(
+        {'conv4_1':Conv2d((1,1),1,strides=1,auto_pad=False,use_bias=True,activation='sigmoid',name='conv4_1'),
+        'conv4_2' :Conv2d((1,1),4,strides=1,auto_pad=False,use_bias=True,name='conv4_2'),
+        'conv4_3':Conv2d((1,1),10,strides=1,auto_pad=False,use_bias=True,name='conv4_3')},is_multicasting = True)
+        ,name='pnet')
 
 
 
@@ -86,10 +88,10 @@ def r_net():
     Flatten(),
     Dense(128,activation=None,use_bias=True,name='conv4'),
     PRelu(num_parameters=1),
-    Combine(
-        Dense(1,activation='sigmoid',use_bias=True,name='conv5_1'),
-        Dense(4,activation=None,use_bias=True,name='conv5_2'),
-        Dense(10,activation=None,use_bias=True,name='conv5_3'))
+    ModuleDict({
+        'conv5_1' :Dense(1,activation='sigmoid',use_bias=True,name='conv5_1'),
+        'conv5_2':Dense(4,activation=None,use_bias=True,name='conv5_2'),
+        'conv5_3':Dense(10,activation=None,use_bias=True,name='conv5_3')},is_multicasting = True)
     ,name='rnet')
 
 
@@ -110,28 +112,30 @@ def o_net():
     Flatten(),
     Dense(256,activation=None,use_bias=True,name='conv5'),
     PRelu(num_parameters=1),
-    Combine(
-        Dense(1,activation='sigmoid',use_bias=True,name='conv6_1'),
-        Dense(4,activation=None,use_bias=True,name='conv6_2'),
-        Dense(10,activation=None,use_bias=True,name='conv6_3')),name='onet')
+    ModuleDict({
+        'conv6_1':Dense(1,activation='sigmoid',use_bias=True,name='conv6_1'),
+        'conv6_2':Dense(4,activation=None,use_bias=True,name='conv6_2'),
+        'conv6_3':Dense(10,activation=None,use_bias=True,name='conv6_3')},is_multicasting = True)
+    ,name='onet')
 
 
 
 def Pnet(pretrained=True,
              input_shape=(3,12,12),
+             freeze_features=False,
              **kwargs):
-    if input_shape is not None and len(input_shape)==3:
-        input_shape=tuple(input_shape)
+    if input_shape is not None and len(input_shape) == 3:
+        input_shape = tuple(input_shape)
     else:
-        input_shape=(3,12,12)
-    pnet =ImageDetectionModel(input_shape=(3,12,12),output=p_net())
-    pnet.preprocess_flow = [normalize(0, 255), image_backend_adaption]
-    if pretrained==True:
-        download_model_from_google_drive('1w9ahipO8D9U1dAXMc2BewuL0UqIBYWSX',dirname,'pnet.pth')
-        recovery_model=load(os.path.join(dirname,'pnet.pth'))
-        recovery_model = fix_layer(recovery_model)
-        recovery_model.to(_device)
-        pnet.model=recovery_model
+        input_shape = (3, 224, 224)
+    pnet = ImageDetectionModel(input_shape=(3,12,12),output=p_net())
+    if pretrained == True:
+        download_model_from_google_drive('1w9ahipO8D9U1dAXMc2BewuL0UqIBYWSX', dirname, 'pnet.pth')
+        recovery_model = fix_layer(load(os.path.join(dirname, 'pnet.pth')))
+        pnet.model = recovery_model
+
+    pnet.model.input_shape = input_shape
+    pnet.model.to(_device)
     return pnet
 
 
@@ -143,7 +147,7 @@ def Rnet(pretrained=True,
     else:
         input_shape=(3,24,24)
     rnet =ImageDetectionModel(input_shape=(3,24,24),output=r_net())
-    rnet.preprocess_flow = [normalize(0, 255), image_backend_adaption]
+    rnet.preprocess_flow = [Normalize(0, 255), image_backend_adaption]
     if pretrained==True:
         download_model_from_google_drive('1CH7z133_KrcWMx9zXAblMCV8luiQ3wph',dirname,'rnet.pth')
         recovery_model=load(os.path.join(dirname,'rnet.pth'))
@@ -160,7 +164,7 @@ def Onet(pretrained=True,
     else:
         input_shape=(3,48,48)
     onet =ImageDetectionModel(input_shape=(3,48,48),output=o_net())
-    onet.preprocess_flow = [normalize(0, 255), image_backend_adaption]
+    onet.preprocess_flow = [Normalize(0, 255), image_backend_adaption]
     if pretrained==True:
         download_model_from_google_drive('1a1dAlSzJOAfIz77Ic38JMQJYWDG_b7-_',dirname,'onet.pth')
         recovery_model=load(os.path.join(dirname,'onet.pth'))
@@ -172,10 +176,10 @@ def Onet(pretrained=True,
 
 
 class DetectorHead(Layer):
-    def __init__(self, cellsize=12,threshould=0.5, min_size=10,**kwargs):
+    def __init__(self, cellsize=12,threshold=0.5, min_size=5,**kwargs):
         super(DetectorHead, self).__init__(**kwargs)
         self.cellsize=cellsize
-        self.threshould=threshould
+        self.threshold=threshold
         self.min_size=min_size
 
         self._built =True
@@ -192,7 +196,7 @@ class DetectorHead(Layer):
         grid=meshgrid(boxprobs.size(1),boxprobs.size(2))
         grid=grid.view(2,-1)
         score = boxprobs[0]
-        y,x = torch.where(score>= self.threshould)
+        y,x = torch.where(score>= self.threshold)
         boxregs = boxregs.permute(1,2,0)
 
         score = score[(y,x )]
@@ -280,11 +284,11 @@ def calibrate_box(bboxes, offsets):
 #
 #         self.signature = get_signature(self._model.forward)
 #         #data preprocess
-#         self.preprocess_flow =[normalize(0,255)]
-#         self.nms_threshould = [0.9, 0.9, 0.3]
-#         self.detection_threshould = [0.5, 0.6, 0.9]
+#         self.preprocess_flow =[Normalize(0,255)]
+#         self.nms_threshold = [0.9, 0.9, 0.3]
+#         self.detection_threshold = [0.5, 0.6, 0.9]
 #
-#         pnet.add_module('pnet_detector', DetectorHead(cellsize=12, threshould=0.5, min_size=self.min_size))
+#         pnet.add_module('pnet_detector', DetectorHead(cellsize=12, threshold=0.5, min_size=self.min_size))
 #
 #
 #     def get_image_pyrimid(self,img,min_size=None,factor= 0.709):
@@ -435,7 +439,7 @@ def calibrate_box(bboxes, offsets):
 #
 #                 #print('total {0} boxes in pnet in all scale '.format(len(boxes)))
 #                 boxes=clip_boxes_to_image(boxes,(img.shape[0],img.shape[1]))
-#                 boxes =self.boxes_nms(boxes, overlap_threshold=self.detection_threshould[0])
+#                 boxes =self.boxes_nms(boxes, overlap_threshold=self.detection_threshold[0])
 #                 if self.verbose:
 #                     print('pnet:{0} boxes '.format(len(boxes)))
 #                 #print('total {0} boxes after nms '.format(len(boxes)))
@@ -450,7 +454,7 @@ def calibrate_box(bboxes, offsets):
 #                         box = boxes[k]
 #                         crop_img = img.copy()[int(box[1]):int(box[3]), int(box[0]):int(box[2]), :]
 #                         if crop_img.shape[0] > 0 and crop_img.shape[1] > 0:
-#                             new_arr[k] = resize((24, 24))(crop_img).transpose([2, 0, 1]) / 255.0
+#                             new_arr[k] = Resize((24, 24))(crop_img).transpose([2, 0, 1]) / 255.0
 #                         # else:
 #                         #     print(box)
 #                     new_arr = to_tensor(new_arr)
@@ -471,7 +475,7 @@ def calibrate_box(bboxes, offsets):
 #                         r_out1, r_out2, r_out3 = self.rnet(new_arr)
 #
 #                     probs =r_out1
-#                     keep =probs[:, 0] > self.detection_threshould[1]
+#                     keep =probs[:, 0] > self.detection_threshold[1]
 #                     r_out1=r_out1[keep]
 #
 #                     boxes = boxes[keep]
@@ -485,7 +489,7 @@ def calibrate_box(bboxes, offsets):
 #                     #######################################
 #                     #########rnet finish
 #                     #######################################
-#                     boxes=self.boxes_nms(boxes, overlap_threshold=self.detection_threshould[1])
+#                     boxes=self.boxes_nms(boxes, overlap_threshold=self.detection_threshold[1])
 #                     if self.verbose:
 #                         print('rnet:{0} boxes '.format(len(boxes)))
 #                     #print('total {0} boxes after nms '.format(len(boxes)))
@@ -498,14 +502,14 @@ def calibrate_box(bboxes, offsets):
 #                         box=boxes[k]
 #                         crop_img=img.copy()[int(box[1]):int(box[3]),int(box[0]):int(box[2]),:]
 #                         if crop_img.shape[0]>0 and crop_img.shape[1]>0:
-#                             new_arr[k]=resize((48,48))(crop_img).transpose([2,0,1])/255.0
+#                             new_arr[k]=Resize((48,48))(crop_img).transpose([2,0,1])/255.0
 #                         # else:
 #                         #     print(box)
 #
 #                     new_arr=to_tensor(new_arr)
 #                     o_out1, o_out2,o_out3  = self.onet(new_arr)
 #                     probs = o_out1
-#                     keep = probs[:, 0] > self.detection_threshould[2]
+#                     keep = probs[:, 0] > self.detection_threshold[2]
 #                     o_out1 = o_out1[keep]
 #                     boxes = boxes[keep]
 #                     if len(boxes)==0:
@@ -524,7 +528,7 @@ def calibrate_box(bboxes, offsets):
 #                     #######################################
 #                     #########onet finish
 #                     #######################################
-#                     boxes = self.boxes_nms(boxes, overlap_threshold=self.detection_threshould[2])
+#                     boxes = self.boxes_nms(boxes, overlap_threshold=self.detection_threshold[2])
 #                     if self.verbose:
 #                         print('onet:{0} boxes '.format(len(boxes)))
 #                     return to_numpy(boxes)
@@ -534,7 +538,7 @@ def calibrate_box(bboxes, offsets):
 #
 #         else:
 #             raise  ValueError('the model is not built yet.')
-#     def generate_bboxes(self,*outputs,threshould=0.5,scale=1):
+#     def generate_bboxes(self,*outputs,threshold=0.5,scale=1):
 #         raise NotImplementedError
 #     def nms(self,bboxes):
 #         raise NotImplementedError
@@ -544,13 +548,14 @@ def calibrate_box(bboxes, offsets):
 class mtcnn(ModuleList):
     def __init__(self, pretrained=True, name='mtcnn'):
         super().__init__(name)
-        self.nms_threshould = [0.9, 0.9, 0.3]
-        self.detection_threshould = [0.5, 0.6, 0.9]
+        self.nms_threshold = [0.9, 0.9, 0.3]
+        self.detection_threshold = [0.5, 0.6, 0.9]
+        self.min_size=10
 
-        self.add_module('pnet',Pnet(pretrained=pretrained))
-        self.add_module('rnet', Rnet(pretrained=pretrained))
-        self.add_module('onet', Onet(pretrained=pretrained))
-        self.add_module('pnet_detector', DetectorHead(cellsize=12, threshould=0.5, min_size=self.min_size))
+        self.add_module('pnet',Pnet(pretrained=pretrained).model)
+        self.add_module('rnet', Rnet(pretrained=pretrained).model)
+        self.add_module('onet', Onet(pretrained=pretrained).model)
+        self.add_module('pnet_detector', DetectorHead(cellsize=12, threshold=0.5, min_size=self.min_size))
 
 
     #adjust bbox like square
@@ -586,7 +591,7 @@ class mtcnn(ModuleList):
 
             # print('total {0} boxes in pnet in all scale '.format(len(boxes)))
             boxes = clip_boxes_to_image(boxes, (x.shape[0], x.shape[1]))
-            boxes = nms(boxes, threshold=self.detection_threshould[0])
+            boxes = nms(boxes, threshold=self.detection_threshold[0])
             print('pnet:{0} boxes '.format(len(boxes)))
             # print('total {0} boxes after nms '.format(len(boxes)))
             # score = to_numpy(boxes[:, 4]).reshape(-1)
@@ -600,7 +605,7 @@ class mtcnn(ModuleList):
                     box = boxes[k]
                     crop_img = x.copy()[int(box[1]):int(box[3]), int(box[0]):int(box[2]), :]
                     if crop_img.shape[0] > 0 and crop_img.shape[1] > 0:
-                        new_arr[k] = resize((24, 24))(crop_img / 255.0).transpose([2, 0, 1])
+                        new_arr[k] = Resize((24, 24))(crop_img / 255.0).transpose([2, 0, 1])
                     # else:
                     #     print(box)
                 new_arr = to_tensor(new_arr)
@@ -621,7 +626,7 @@ class mtcnn(ModuleList):
                     r_out1, r_out2, r_out3 = self.rnet(new_arr)
 
                 probs = to_numpy(r_out1)
-                keep = np.where(probs[:, 0] > self.detection_threshould[1])[0]
+                keep = np.where(probs[:, 0] > self.detection_threshold[1])[0]
                 r_out1 = r_out1[keep]
                 boxes = boxes[keep]
                 boxes[:, 4] = r_out1[:, 0]
@@ -632,7 +637,7 @@ class mtcnn(ModuleList):
                 #########rnet finish
                 #######################################
 
-                boxes = nms(boxes, threshold=self.detection_threshould[1], image_size=(x.shape[0], x.shape[1]), min_size=self.min_size)
+                boxes = nms(boxes, threshold=self.detection_threshold[1], image_size=(x.shape[0], x.shape[1]), min_size=self.min_size)
                 print('rnet:{0} boxes '.format(len(boxes)))
                 # print('total {0} boxes after nms '.format(len(boxes)))
                 boxes = clip_boxes_to_image(boxes, (x.shape[0], x.shape[1]))
@@ -643,14 +648,14 @@ class mtcnn(ModuleList):
                     box = boxes[k]
                     crop_img = x.copy()[int(box[1]):int(box[3]), int(box[0]):int(box[2]), :]
                     if crop_img.shape[0] > 0 and crop_img.shape[1] > 0:
-                        new_arr[k] = resize((48, 48))(crop_img / 255.0).transpose([2, 0, 1])
+                        new_arr[k] = Resize((48, 48))(crop_img / 255.0).transpose([2, 0, 1])
                     # else:
                     #     print(box)
 
                 new_arr = to_tensor(new_arr)
                 o_out1, o_out2, o_out3 = self.onet(new_arr)
                 probs = to_numpy(o_out1)
-                keep = np.where(probs[:, 0] > self.detection_threshould[2])[0]
+                keep = np.where(probs[:, 0] > self.detection_threshold[2])[0]
                 o_out1 = o_out1[keep]
                 boxes = boxes[keep]
 
@@ -673,7 +678,7 @@ class mtcnn(ModuleList):
 class Mtcnn(ImageDetectionModel):
     def __init__(self, pretrained=True, min_size=10, **kwargs):
         model=mtcnn(pretrained=pretrained)
-        super(Mtcnn, self).__init__(input_shape=(12, 12,3), output=model)
+        super(Mtcnn, self).__init__(input_shape=(3,12, 12), output=model)
         self.min_size = min_size
         self.signature=get_signature(self.model.forward)
         self.preprocess_flow =[normalize(0,255)]
@@ -723,13 +728,13 @@ class Mtcnn(ImageDetectionModel):
                 boxes_list.append(boxes)
             boxes=concate(boxes_list,axis=0)
             #idx=int(np.argmax(result,-1)[0])
-            boxes = nms(boxes, threshold=self.detection_threshould[2], image_size=(img.shape[0], img.shape[1]), min_size=self.min_size)
+            boxes = nms(boxes, threshold=self.detection_threshold[2], image_size=(img.shape[0], img.shape[1]), min_size=self.min_size)
             print('onet:{0} boxes '.format(len(boxes)))
             return boxes
         else:
             return img
 
-    def generate_bboxes(self,*outputs,threshould=0.5,scale=1):
+    def generate_bboxes(self,*outputs,threshold=0.5,scale=1):
         raise NotImplementedError
     def nms(self,bboxes):
         raise NotImplementedError
