@@ -79,7 +79,7 @@ class TrainingPlan(object):
         self.save_model_frequency = -1
         self.save_model_unit = 'batch'
         self.execution_id = None
-        self.enable_tensorboard = False
+        self.enable_tensorboard = ctx.enable_tensorboard
         self._is_optimizer_warmup = False
 
         self.callbacks = []  # if self.callbacks is None:  #     self.callbacks = [  #
@@ -206,22 +206,22 @@ class TrainingPlan(object):
         training_item.training_context['training_name'] = self.training_names[n]
         self.training_items[n].start_epoch = start_epoch
         # backward compatibility
-        for k, v in training_item.inputs.items():
-            if isinstance(v, tuple) and all([isinstance(item, numbers.Integral) for item in v]):
-                training_item.inputs[k] = TensorSpec(shape=to_tensor(v), name=training_item.name)
-                training_item.signature.inputs[k] = TensorSpec(shape=to_tensor(v), name=training_item.name)
-            elif isinstance(v, TensorSpec):
-                training_item.signature.inputs[k] = v
-
-        for k, v in training_item.outputs.items():
-            if isinstance(v, tuple) and all([isinstance(item, numbers.Integral) for item in v]):
-                training_item.outputs[k] = TensorSpec(shape=to_tensor(v), name=training_item.name)
-                training_item.signature.outputs[k] = TensorSpec(shape=to_tensor(v), name=training_item.name)
-            elif isinstance(v, TensorSpec):
-                training_item.signature.outputs[k] = v
-        if isinstance(training_item.model, Layer) and training_item.signature != training_item.model.signature:
-            training_item.model.signature = None
-            training_item.signature = training_item.model.signature
+        # for k, v in training_item.signature.inputs.items():
+        #     if isinstance(v, tuple) and all([isinstance(item, numbers.Integral) for item in v]):
+        #         training_item.inputs[k] = TensorSpec(shape=to_tensor(v), name=training_item.name)
+        #         training_item.signature.inputs[k] = TensorSpec(shape=to_tensor(v), name=training_item.name)
+        #     elif isinstance(v, TensorSpec):
+        #         training_item.signature.inputs[k] = v
+        #
+        # for k, v in training_item.signature.outputs.items():
+        #     if isinstance(v, tuple) and all([isinstance(item, numbers.Integral) for item in v]):
+        #         training_item.outputs[k] = TensorSpec(shape=to_tensor(v), name=training_item.name)
+        #         training_item.signature.outputs[k] = TensorSpec(shape=to_tensor(v), name=training_item.name)
+        #     elif isinstance(v, TensorSpec):
+        #         training_item.signature.outputs[k] = v
+        # if isinstance(training_item.model, Layer) and training_item.signature != training_item.model.signature:
+        #     training_item.model.signature = None
+        #     training_item.signature = training_item.model.signature
         return self
 
     def with_data_loader(self, data_loader, **kwargs):
@@ -243,12 +243,12 @@ class TrainingPlan(object):
 
     @deprecated('0.7.0', 'with_tensorboard')
     def within_tensorboard(self):
-        self.enable_tensorboard = True
+        make_dir_if_need(os.path.join(working_directory, 'Logs'))
         # check weather have tensorboard
         if get_backend() == 'pytorch':
             try:
                 from trident.loggers.pytorch_tensorboard import SummaryWriter
-                self.summary_writer = SummaryWriter(os.path.join(working_directory, 'Logs'))
+                ctx.try_enable_tensorboard(SummaryWriter(os.path.join(working_directory, 'Logs')))
 
             except Exception as e:
                 print('Tensorboard initialize failed, please check the installation status about Tensorboard.')
@@ -257,7 +257,8 @@ class TrainingPlan(object):
         elif get_backend() == 'tensorflow':
             try:
                 from trident.loggers.tensorflow_tensorboard import SummaryWriter
-                self.summary_writer = SummaryWriter(os.path.join(working_directory, 'Logs'))
+                ctx.try_enable_tensorboard(SummaryWriter(os.path.join(working_directory, 'Logs')))
+
             except Exception as e:
                 print('Tensorboard initialize failed, please check the installation status about Tensorboard.')
                 print(e)
@@ -265,7 +266,6 @@ class TrainingPlan(object):
         return self
 
     def with_tensorboard(self):
-        self.enable_tensorboard = True
         make_dir_if_need(os.path.join(working_directory, 'Logs'))
         # check weather have tensorboard
         if get_backend() == 'pytorch':
@@ -315,7 +315,7 @@ class TrainingPlan(object):
 
     def print_gradients_scheduling(self, frequency: int, unit='batch'):
 
-        pg = PrintGradientsCallback(batch_inteval=frequency)
+        pg = PrintGradientsCallback(frequency=frequency,unit=unit,)
         pg.is_shared = False
         self.callbacks.append(pg)
         return self
@@ -338,7 +338,7 @@ class TrainingPlan(object):
         if unit not in ['batch', 'epoch']:
             raise ValueError('unit should be batch or epoch')
 
-        tile = TileImageCallback(frequency if unit == 'epoch' else -1, frequency if unit == 'batch' else -1,
+        tile = TileImageCallback(frequency=frequency,unit=unit,
                                  save_path=save_path, name_prefix=name_prefix, include_input=include_input,
                                  include_output=include_output, include_target=include_target,
                                  include_mask=include_mask, imshow=imshow)
@@ -362,7 +362,7 @@ class TrainingPlan(object):
                 except Exception as e:
                     PrintException()
                     raise ValueError('save_path:{0} is not valid path'.format(folder))
-        plot = PlotLossMetricsCallback(frequency if unit == 'epoch' else -1, frequency if unit == 'batch' else -1,
+        plot = PlotLossMetricsCallback(frequency=frequency,unit=unit,
                                        save_path=save_path, name_prefix=name_prefix.format(get_time_suffix()),
                                        clean_ipython_output_frequency=clean_ipython_output_frequency, imshow=imshow)
         self.callbacks.append(plot)
@@ -442,7 +442,9 @@ class TrainingPlan(object):
                 #     available_items.remove(out.replace("output","target").replace("student","teacher"))
 
             trainingitem.training_context['data_feed'] = data_feed
-            print('data_feed for {0} :{1}'.format(trainingitem.name, data_feed))
+            print('data_feed for {0} :'.format(trainingitem.name))
+            print(json.dumps(data_feed, indent=4, sort_keys=True))
+
 
     def start_now(self, collect_data_inteval=1, is_resume=False, only_steps=False, max_batches=np.inf,
                   keep_weights_history=False, keep_gradient_history=False):
@@ -454,7 +456,7 @@ class TrainingPlan(object):
             exception_cnt = 0
             abnormal_num_count = 0
             # update callback
-            if self.enable_tensorboard:
+            if ctx.enable_tensorboard:
                 for idx, (item, item_name) in enumerate(zip(self.training_items.value_list, self.training_names.value_list)):
                     if hasattr(item, 'training_context'):
                         for context_item in list(item.training_context.values()):
@@ -536,7 +538,7 @@ class TrainingPlan(object):
                                 need_out_sample_evaluation = True
 
                             iter_testdata = None
-                            if isinstance(data_provider, DataProvider) and data_provider.testdata is not None and need_out_sample_evaluation:
+                            if isinstance(data_provider, (DataProvider,TextSequenceDataProvider)) and data_provider.testdata is not None and need_out_sample_evaluation:
                                 return_test = data_provider.next_test()
                                 if return_test is not None:
                                     iter_testdata = OrderedDict()
@@ -555,22 +557,36 @@ class TrainingPlan(object):
                                 trainitem.training_context['data_template'] = data_provider.traindata.data_template
                                 trainitem.training_context['collect_data_inteval'] = collect_data_inteval
                                 trainitem.training_context['model_name'] = trainitem_name
-                                if epoch < int(trainitem.start_epoch):
+                                start_epoch=0 if not hasattr(trainitem,'start_epoch') else trainitem.start_epoch
+                                if epoch <start_epoch:
                                     trainitem.training_context['stop_update'] = 1
 
+                                num_batches=len(data_provider.batch_sampler) if only_steps == False else max_batches
+                                current_batch=mbs if only_steps == False else self.steps
+                                current_epoch=epoch if only_steps == False else 0
+                                if current_batch ==0:
+                                    if current_epoch==0:
+                                        trainitem.do_on_training_start()
+                                    trainitem.do_on_epoch_start()
+                                trainitem.steps=self.steps
+                                trainitem.current_epoch=current_epoch
+                                trainitem.current_batch=current_batch
                                 trainitem.train_model(train_data, test_data,
                                                       epoch if only_steps == False else 0,
                                                       mbs if only_steps == False else self.steps,
                                                       self.num_epochs if only_steps == False else 1,
-                                                      len(data_provider.batch_sampler) if only_steps == False else max_batches,
+                                                      num_batches,
+                                                      done=None,
                                                       is_collect_data=mbs == 0 or mbs % collect_data_inteval == 0,
                                                       is_print_batch_progress=self.print_progress_unit == 'batch' and mbs > 0 and mbs % self.print_progress_frequency == 0,
                                                       is_print_epoch_progress=self.print_progress_unit == 'epoch' and epoch > 0 and epoch % self.print_progress_frequency == 0,
                                                       log_gradients=keep_gradient_history, log_weights=keep_weights_history,
                                                       accumulate_grads=(trainitem.training_context['steps']+1) % trainitem.accumulation_steps != 0, is_out_sample_evaluation=need_out_sample_evaluation)
+                                if current_batch==num_batches-1:
+                                    trainitem.do_on_epoch_end()
                             self.steps += 1
 
-                            if self.enable_tensorboard and len(self.training_items) > 1 and mbs % collect_data_inteval == 0:
+                            if ctx.enable_tensorboard and len(self.training_items) > 1 and mbs % collect_data_inteval == 0:
                                 compare_dict = OrderedDict()
                                 step = None
                                 for trainitem_name, trainitem in zip(self.training_names.value_list, self.training_items.value_list):
@@ -605,15 +621,15 @@ class TrainingPlan(object):
                             if self.save_model_frequency > 0 and self.save_model_unit == 'batch' and (self.steps + 1) % \
                                     self.save_model_frequency == 0:
                                 for k, trainitem in self.training_items.items():
-                                    trainitem.save_model(trainitem.training_context['save_path'])
-                                    if self.enable_tensorboard and ('upload_onnx' not in trainitem.training_context or trainitem.training_context['upload_onnx'] == False):
+                                    trainitem.save_model(trainitem.training_context['save_path'], )
+                                    if ctx.enable_tensorboard and ('upload_onnx' not in trainitem.training_context or trainitem.training_context['upload_onnx'] == False):
                                         trainitem.save_onnx(trainitem.training_context['save_path'].replace('.pth', '.onnx'))
-                                        self.summary_writer.add_onnx_graph(trainitem.training_context['save_path'].replace('.pth', '.onnx'));
+                                        ctx.summary_writer.add_onnx_graph(trainitem.training_context['save_path'].replace('.pth', '.onnx'));
                                         trainitem.training_context['upload_onnx'] = True
                             if only_steps == True and self.steps >= max_batches - 1:
                                 for k, trainitem in self.training_items.items():
                                     try:
-                                        trainitem.save_model(trainitem.training_context['save_path'])
+                                        trainitem.save_model(trainitem.training_context['save_path'], )
                                     except Exception as e:
                                         print(e)
                                 data_provider.mode = 'tuple'
@@ -626,36 +642,36 @@ class TrainingPlan(object):
                 except StopIteration:
                     for k, trainitem in self.training_items.items():
                         trainitem.do_on_epoch_end()
-                        trainitem.save_model(trainitem.training_context['save_path'])
+                        trainitem.save_model(trainitem.training_context['save_path'], )
 
                 except ValueError as ve:
                     print(ve)
                     PrintException()
                     for k, trainitem in self.training_items.items():
                         trainitem.do_on_training_end()
-                        trainitem.save_model(trainitem.training_context['save_path'])
+                        trainitem.save_model(trainitem.training_context['save_path'], )
                 except Exception as e:
                     print(e)
                     PrintException()
                     for k, trainitem in self.training_items.items():
                         trainitem.do_on_training_end()
-                        trainitem.save_model(trainitem.training_context['save_path'])
+                        trainitem.save_model(trainitem.training_context['save_path'], )
                 if self.save_model_frequency > 0 and self.save_model_unit == 'epoch' and (
                         epoch + 1) % self.save_model_frequency == 0:
                     for k, trainitem in self.training_items.items():
-                        trainitem.save_model(trainitem.training_context['save_path'])
+                        trainitem.save_model(trainitem.training_context['save_path'], )
             data_provider.mode = 'tuple'
 
 
         except KeyboardInterrupt:
             for k, trainitem in self.training_items.items():
-                trainitem.save_model(trainitem.training_context['save_path'])
+                trainitem.save_model(trainitem.training_context['save_path'], )
             data_provider.mode = 'tuple'
         except Exception as e:
             print(e)
             PrintException()
             for k, trainitem in self.training_items.items():
-                trainitem.save_model(trainitem.training_context['save_path'])
+                trainitem.save_model(trainitem.training_context['save_path'], )
             data_provider.mode = 'tuple'
 
     def resume(self):
@@ -1104,7 +1120,7 @@ class GanTrainingPlan(TrainingPlan):
             exception_cnt = 0
             abnormal_num_count = 0
             # update callback
-            if self.enable_tensorboard:
+            if ctx.enable_tensorboard:
                 for idx, (item, item_name) in enumerate(zip(self.training_items.value_list, self.training_names.value_list)):
                     if hasattr(item, 'training_context'):
                         for context_item in list(item.training_context.values()):
@@ -1250,7 +1266,7 @@ class GanTrainingPlan(TrainingPlan):
                                                       accumulate_grads=False, is_out_sample_evaluation=need_out_sample_evaluation)
                             self.steps += 1
 
-                            if self.enable_tensorboard and len(self.training_items) > 1 and mbs % collect_data_inteval == 0:
+                            if ctx.enable_tensorboard and len(self.training_items) > 1 and mbs % collect_data_inteval == 0:
                                 compare_dict = OrderedDict()
                                 step = None
                                 for trainitem_name, trainitem in zip(self.training_names.value_list, self.training_items.value_list):
@@ -1282,15 +1298,15 @@ class GanTrainingPlan(TrainingPlan):
                             if self.save_model_frequency > 0 and self.save_model_unit == 'batch' and (self.steps + 1) % \
                                     self.save_model_frequency == 0:
                                 for k, trainitem in self.training_items.items():
-                                    trainitem.save_model(trainitem.training_context['save_path'])
-                                    if self.enable_tensorboard and ('upload_onnx' not in trainitem.training_context or trainitem.training_context['upload_onnx'] == False):
+                                    trainitem.save_model(trainitem.training_context['save_path'], )
+                                    if ctx.enable_tensorboard and ('upload_onnx' not in trainitem.training_context or trainitem.training_context['upload_onnx'] == False):
                                         trainitem.save_onnx(trainitem.training_context['save_path'].replace('.pth', '.onnx'))
-                                        self.summary_writer.add_onnx_graph(trainitem.training_context['save_path'].replace('.pth', '.onnx'));
+                                        ctx.summary_writer.add_onnx_graph(trainitem.training_context['save_path'].replace('.pth', '.onnx'));
                                         trainitem.training_context['upload_onnx'] = True
                             if only_steps == True and self.steps >= max_batches - 1:
                                 for k, trainitem in self.training_items.items():
                                     try:
-                                        trainitem.save_model(trainitem.training_context['save_path'])
+                                        trainitem.save_model(trainitem.training_context['save_path'], )
                                     except Exception as e:
                                         print(e)
                                 data_provider.mode = 'tuple'
@@ -1303,36 +1319,36 @@ class GanTrainingPlan(TrainingPlan):
                 except StopIteration:
                     for k, trainitem in self.training_items.items():
                         trainitem.do_on_epoch_end()
-                        trainitem.save_model(trainitem.training_context['save_path'])
+                        trainitem.save_model(trainitem.training_context['save_path'], )
 
                 except ValueError as ve:
                     print(ve)
                     PrintException()
                     for k, trainitem in self.training_items.items():
                         trainitem.do_on_training_end()
-                        trainitem.save_model(trainitem.training_context['save_path'])
+                        trainitem.save_model(trainitem.training_context['save_path'], )
                 except Exception as e:
                     print(e)
                     PrintException()
                     for k, trainitem in self.training_items.items():
                         trainitem.do_on_training_end()
-                        trainitem.save_model(trainitem.training_context['save_path'])
+                        trainitem.save_model(trainitem.training_context['save_path'], )
                 if self.save_model_frequency > 0 and self.save_model_unit == 'epoch' and (
                         epoch + 1) % self.save_model_frequency == 0:
                     for k, trainitem in self.training_items.items():
-                        trainitem.save_model(trainitem.training_context['save_path'])
+                        trainitem.save_model(trainitem.training_context['save_path'], )
             data_provider.mode = 'tuple'
 
 
         except KeyboardInterrupt:
             for k, trainitem in self.training_items.items():
-                trainitem.save_model(trainitem.training_context['save_path'])
+                trainitem.save_model(trainitem.training_context['save_path'], )
             data_provider.mode = 'tuple'
         except Exception as e:
             print(e)
             PrintException()
             for k, trainitem in self.training_items.items():
-                trainitem.save_model(trainitem.training_context['save_path'])
+                trainitem.save_model(trainitem.training_context['save_path'], )
             data_provider.mode = 'tuple'
 
 
